@@ -98,6 +98,7 @@ modules = [
     "lexica",
     "lightroom",
     "livedoor",
+    "lofter",
     "luscious",
     "lynxchan",
     "mangadex",
@@ -195,6 +196,7 @@ modules = [
     "wikimedia",
     "xhamster",
     "xvideos",
+    "yiffverse",
     "zerochan",
     "zzup",
     "booru",
