@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2022 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -24,11 +24,11 @@ class NozomiExtractor(Extractor):
     filename_fmt = "{postid} {dataid}.{extension}"
     archive_fmt = "{dataid}"
 
-    def items(self):
-
-        data = self.metadata()
+    def _init(self):
         self.session.headers["Origin"] = self.root
-        self.session.headers["Referer"] = self.root + "/"
+
+    def items(self):
+        data = self.metadata()
 
         for post_id in map(str, self.posts()):
             url = "https://j.nozomi.la/post/{}/{}/{}.json".format(
@@ -62,10 +62,22 @@ class NozomiExtractor(Extractor):
 
             yield Message.Directory, post
             for post["num"], image in enumerate(images, 1):
-                post["url"] = url = text.urljoin(self.root, image["imageurl"])
-                text.nameext_from_url(url, post)
-                post["is_video"] = bool(image.get("is_video"))
-                post["dataid"] = post["filename"]
+                post["filename"] = post["dataid"] = did = image["dataid"]
+                post["is_video"] = video = \
+                    True if image.get("is_video") else False
+
+                ext = image["type"]
+                if video:
+                    subdomain = "v"
+                elif ext == "gif":
+                    subdomain = "g"
+                else:
+                    subdomain = "w"
+                    ext = "webp"
+
+                post["extension"] = ext
+                post["url"] = url = "https://{}.nozomi.la/{}/{}/{}.{}".format(
+                    subdomain, did[-1], did[-3:-1], did, ext)
                 yield Message.Url, url, post
 
     def posts(self):
@@ -94,39 +106,7 @@ class NozomiPostExtractor(NozomiExtractor):
     """Extractor for individual posts on nozomi.la"""
     subcategory = "post"
     pattern = r"(?:https?://)?nozomi\.la/post/(\d+)"
-    test = (
-        ("https://nozomi.la/post/3649262.html", {
-            "url": "f4522adfc8159355fd0476de28761b5be0f02068",
-            "content": "cd20d2c5149871a0b80a1b0ce356526278964999",
-            "keyword": {
-                "artist"   : ["hammer (sunset beach)"],
-                "character": ["patchouli knowledge"],
-                "copyright": ["touhou"],
-                "dataid"   : "re:aaa9f7c632cde1e1a5baaff3fb6a6d857ec73df7fdc5",
-                "date"     : "dt:2016-07-26 02:32:03",
-                "extension": "jpg",
-                "filename" : str,
-                "height"   : 768,
-                "is_video" : False,
-                "postid"   : 3649262,
-                "source"   : "danbooru",
-                "tags"     : list,
-                "type"     : "jpg",
-                "url"      : str,
-                "width"    : 1024,
-            },
-        }),
-        #  multiple images per post
-        ("https://nozomi.la/post/25588032.html", {
-            "url": "6aa3b7db385abcc9d374bdffd19187bccbf8f228",
-            "keyword": "f60e048df36308b6b25dfaac419b586895d360bc",
-            "count": 7,
-        }),
-        # empty 'date' (#1163)
-        ("https://nozomi.la/post/130309.html", {
-            "keyword": {"date": None},
-        })
-    )
+    example = "https://nozomi.la/post/12345.html"
 
     def __init__(self, match):
         NozomiExtractor.__init__(self, match)
@@ -141,11 +121,7 @@ class NozomiIndexExtractor(NozomiExtractor):
     subcategory = "index"
     pattern = (r"(?:https?://)?nozomi\.la/"
                r"(?:(index(?:-Popular)?)-(\d+)\.html)?(?:$|#|\?)")
-    test = (
-        ("https://nozomi.la/"),
-        ("https://nozomi.la/index-2.html"),
-        ("https://nozomi.la/index-Popular-33.html"),
-    )
+    example = "https://nozomi.la/index-1.html"
 
     def __init__(self, match):
         NozomiExtractor.__init__(self, match)
@@ -159,11 +135,7 @@ class NozomiTagExtractor(NozomiExtractor):
     directory_fmt = ("{category}", "{search_tags}")
     archive_fmt = "t_{search_tags}_{dataid}"
     pattern = r"(?:https?://)?nozomi\.la/tag/([^/?#]+)-(\d+)\."
-    test = ("https://nozomi.la/tag/3:1_aspect_ratio-1.html", {
-        "pattern": r"^https://i.nozomi.la/\w/\w\w/\w+\.\w+$",
-        "count": ">= 25",
-        "range": "1-25",
-    })
+    example = "https://nozomi.la/tag/TAG-1.html"
 
     def __init__(self, match):
         NozomiExtractor.__init__(self, match)
@@ -181,9 +153,7 @@ class NozomiSearchExtractor(NozomiExtractor):
     directory_fmt = ("{category}", "{search_tags:J }")
     archive_fmt = "t_{search_tags}_{dataid}"
     pattern = r"(?:https?://)?nozomi\.la/search\.html\?q=([^&#]+)"
-    test = ("https://nozomi.la/search.html?q=hibiscus%203:4_ratio#1", {
-        "count": ">= 5",
-    })
+    example = "https://nozomi.la/search.html?q=QUERY"
 
     def __init__(self, match):
         NozomiExtractor.__init__(self, match)
@@ -193,25 +163,28 @@ class NozomiSearchExtractor(NozomiExtractor):
         return {"search_tags": self.tags}
 
     def posts(self):
-        index = None
-        result = set()
+        result = None
+        positive = []
+        negative = []
 
         def nozomi(path):
             url = "https://j.nozomi.la/" + path + ".nozomi"
             return decode_nozomi(self.request(url).content)
 
         for tag in self.tags:
-            tag = tag.replace("/", "")
-            if tag[0] == "-":
-                if not index:
-                    index = set(nozomi("index"))
-                items = index.difference(nozomi("nozomi/" + tag[1:]))
-            else:
-                items = nozomi("nozomi/" + tag)
+            (negative if tag[0] == "-" else positive).append(
+                tag.replace("/", ""))
 
-            if result:
-                result.intersection_update(items)
+        for tag in positive:
+            ids = nozomi("nozomi/" + tag)
+            if result is None:
+                result = set(ids)
             else:
-                result.update(items)
+                result.intersection_update(ids)
 
-        return sorted(result, reverse=True)
+        if result is None:
+            result = set(nozomi("index"))
+        for tag in negative:
+            result.difference_update(nozomi("nozomi/" + tag[1:]))
+
+        return sorted(result, reverse=True) if result else ()
